library(pacman)
p_load(data.table, foreach, tidyverse,         # main
               ggpubr, knitr, kableExtra,      # visuals
               broom,                          # stats
               DT, plotly, shiny)              # web


source("/scratch/rsrc/utils.R", chdir = TRUE)
set.seed(1234)

# knitr::opts_knit$set(
#   root.dir = getwd()
# )
knitr::opts_chunk$set(
  context   = "render",
  echo      = FALSE, 
  include   = TRUE, 
  out.width = "100%", 
  fig.width = 10, 
  fig.height= 6)





################
# CACHING
################

RECOMPUTE <<- FALSE
touch <- function() {
    RECOMPUTE <<- TRUE
}
cache <- function(foo = foo, fname, ...,
    condition = RECOMPUTE, verbose = FALSE, 
    path = "../output/", returnResult = TRUE) {


    # If condition evaluates to TRUE, then 
    # recompute using the call to function
    if (!grepl("[Rr][Dd][Ss]", fname)) {
        fname <- paste0(fname, ".RDS")
    }
    if (!file.exists(path)) {
        message("Directory doesn't exist - creating")
        dir.create(path, recursive = TRUE)
    }
    fname <- file.path(path, fname)
    if (condition | !file.exists(fname)) {
        if (verbose) {
            message("Recomputing.")
        }
        res <- foo(...)
        saveRDS(res, fname, compress = FALSE)
        RECOMPUTE <<- FALSE
    } else {
        if (verbose) {
            message("Using cached result.")
        }
        if (returnResult) {
            res <- readRDS(fname)
        }
    }
    if (returnResult) {
        return(res)
    } else {
        return(NULL)
    }
}
